package com.anpgames.beans.session;

import java.io.Serializable;
import java.util.List;

import javax.ejb.EJB;
import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.anpgames.beans.application.PermissionsSupportMB;
import com.anpgames.slsbeans.basic.user.UserBeanLocal;

@ManagedBean
@SessionScoped
public class UserMB implements Serializable {
	private static final long serialVersionUID = -6724328460083080044L;
	private String login;
	private String password;
	private boolean loggedIn;
	private List<Integer> permissions;
	
	@EJB UserBeanLocal userBean;
	@ManagedProperty(value="#{permissionsSupportMB}")
	private PermissionsSupportMB permSupportMB;
	
	public void setPermSupportMB(PermissionsSupportMB psMB) {
		this.permSupportMB = psMB;
	}
	
	public void setLogin(String login) {
		this.login = login;
	}
	public String getLogin() {
		return login;
	}
	
	public void setPassword(String pass) {
		this.password = pass;
	}
	public String getPassword() {
		return password;
	}
	
	public boolean isLoggedIn() {
		return loggedIn;
	}
	
	/**
	 * Metoda wywoływana gdy użytkownik loguje się do aplikacji.
	 * @return outcome
	 */
	public String logIn() {
		if (userBean.tryLogIn(login, password)) {
			loggedIn = true;
			password = null;
			
			permissions = userBean.getUserPermissions(login);
			userBean.tmpMethod();
		}
		return null;
	}
	
	/**
	 * Metoda wywoływana w momencie wylogowywania się z aplikacji przez użytkownika.
	 * @return outcome
	 */
	public String logOut() {
		loggedIn = false;
		HttpSession session = (HttpSession) FacesContext.getCurrentInstance().getExternalContext().getSession(false);
		session.invalidate();
//		throw new NullPointerException("nie bangla;/");
		
		return "logout";
	}
	
	/**
	 * Metoda weryfikująca czy użytkownik ma dostęp do zasobu określonego przez parametr.
	 * @param permissionNumber numer uprawnienia do weryfikacji
	 * @return <code>true</code> - użytkownik posiada uprawnienia, <code>false</code> - żytkownik nie ma uprawnień
	 */
	public boolean isAuthorized(int permissionNumber) {
		if (!loggedIn)
			return false;
		
		if (permissions.contains(permissionNumber))
			return true;
		
		return false;
	}
	
	/**
	 * Metoda weryfikująca uprawnienia użytkownika do konkretnych stron.
	 * Uprawnienia są weryfikowane jedynie w przypadku żądań typu <code>GET</code>. 
	 * SYSTEM EVENT: <code>PreRenderView</code>
	 * @param e obiekt zdarzenia
	 */
	public void checkPermissions(ComponentSystemEvent e) {
		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
		ConfigurableNavigationHandler handler = (ConfigurableNavigationHandler) context.getApplication().getNavigationHandler();
		
		if (request.getMethod().equalsIgnoreCase("POST"))
			return;
		
		//TODO dla szybkiego testowania wykluczeni userzy
//		if (login==null || login.equals("asd") || login.equals("qwe"))
//			return;
		
		//sprawdzanie czy uzytkownik jest zalogowany 
		if (!loggedIn)
			handler.performNavigation("index");
		
		String requestURI = request.getRequestURI();
		
		//sprawdzenie czy uzytkownik
		if (requestURI.contains(PermissionsSupportMB.MODULES_PATH)) {
			if (!hasModulePermission(requestURI)
					|| !hasPagePermission(requestURI))
				handler.performNavigation("notAuthorized");
		}
	}
	
	/**
	 * Metoda ogólnej weryfikacji, sprawdzająca czy użytkownik ma dostęp
	 * do żądanego modułu.
	 * @param requestURI ścieżka do żądanego zasobu
	 * @return <code>true</code> lub <code>false</code>
	 */
	public boolean hasModulePermission(String requestURI) {
		boolean hasPermission = false;
		if (requestURI.contains(PermissionsSupportMB.M_ADMIN_PATH)) {
			if (isAuthorized(PermissionsSupportMB.M_ADMIN_PERM_NUM))
				hasPermission = true;
		}
		else if(requestURI.contains(PermissionsSupportMB.M_REPORT_PATH)) {
			if (isAuthorized(PermissionsSupportMB.M_REPORT_PERM_NUM))
				hasPermission = true;
		}
		
		return hasPermission;
	}
	
	/**
	 * Metoda szczegółowej weryfikacji, sprawdzająca czy użytkownik ma dostęp
	 * do żądanej strony w konkretnym module.
	 * Jeżeli strona nie znajduje się na liście stron wymagających uprawnień do przeglądania,
	 * metoda zwraca wartość <code>true</code> zezwalając na dostęp do żądanej strony.
	 * @param requestURI ścieżka do żądanego zasobu
	 * @return <code>true</code> lub <code>false</code>
	 */
	public boolean hasPagePermission(String requestURI) {
		String pagePath = requestURI.substring(requestURI.indexOf(PermissionsSupportMB.MODULES_PATH));
		int pagePermNumber = permSupportMB.getPagePermNumber(pagePath);

		//wartosc -1 oznacza, ze strona nie podlega autoryzacji
		if (pagePermNumber == -1)
			return true;
		
		if (isAuthorized(pagePermNumber))
			return true;
		
		return false;
	}
	
}
