package com.anpgames.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import com.anpgames.modules.admin.entities.Group;

/**
 * Entity implementation class for Entity: User
 *
 */
@Entity
@Table(name="APP_USERS")
public class User implements Serializable {
	private static final long serialVersionUID = 2171158761894526407L;
	
	@Id
	@GeneratedValue
	@Column(name="USER_ID")
	private Integer userId;
	
	@Column(name="LOGIN", length=30, nullable=false, unique=true)
	private String login;
	
	@Column(name="PASSWORD", length=40)
	private String password;
	
	@Temporal(TemporalType.DATE)
	@Column(name="CREATION_DATE")
	private Date creationDate;
	
	@Temporal(TemporalType.DATE)
	@Column(name="LAST_UPDATE_DATE")
	private Date lastUpdateDate;
	
	@ManyToMany(fetch=FetchType.LAZY)
	@JoinTable(name="APP_USERS_GROUPS",
			joinColumns=@JoinColumn(name="USER_ID"),
			inverseJoinColumns=@JoinColumn(name="GROUP_ID"))
	private List<Group> groups;
	
	
	public User() {
		super();
	}
	
	
	public Integer getUserId() {
		return userId;
	}
	public void setUserId(Integer user_id) {
		this.userId = user_id;
	}

	
	public String getLogin() {
		return login;
	}
	public void setLogin(String login) {
		this.login = login;
	}

	
	public String getPassword() {
		return password;
	}
	public void setPassword(String pass) {
		this.password = pass;
	}

	
	public Date getCreationDate() {
		return creationDate;
	}
	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	
	public Date getLastUpdateDate() {
		return lastUpdateDate;
	}
	public void setLastUpdateDate(Date lastUpdateDate) {
		this.lastUpdateDate = lastUpdateDate;
	}

	
	public void setGroups(List<Group> groups) {
		this.groups = groups;
	}
	public List<Group> getGroups() {
		return groups;
	}
	
	
	/**
	 * Metoda wykorzystywana do aktualizcji głównych pól encji w oparciu o właściwości innego obiektu tej samej klasy.
	 * @param u obiekt, z którego pobieranę są nowe wartości pól
	 */
   public void setMainFields(User u) {
   	this.login = u.getLogin();
   	this.password = u.getPassword();
   	this.lastUpdateDate = new Date();
   }
}
