package com.anpgames.slsbeans.basic.user;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import com.anpgames.entities.User;
import com.anpgames.support.HashAlghoritms;

/**
 * Session Bean implementation class UserBean
 */
@Stateless
public class UserBean implements UserBeanLocal {
	@PersistenceContext(unitName="tsPU") 
	private EntityManager manager;

	/**
	 * Default constructor.
	 */
	public UserBean() {
		// TODO Auto-generated constructor stub
	}
	
	@Override
	public void createUser(User u) {
		manager.persist(u);
	}
	
	@Override
	public void updateUser(User u) {
		User user = manager.find(User.class, u.getUserId());
		user.setLogin(u.getLogin());
		manager.flush();
	}
	
	@Override
	public User findUser(int pKey) {
		return manager.find(User.class, pKey);
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public List<User> getUsers() {
		List<User> list = null;
		String q = "SELECT u FROM User u";
		Query query = manager.createQuery(q);
		list = query.getResultList();
		
		return list;
	}
	
	@Override
	public void tmpMethod() {
		String q = "SELECT login, password FROM APP_USERS";
		Query query = manager.createNativeQuery(q);
		List results = query.getResultList();
		Iterator it = results.iterator();
		while (it.hasNext()) {
			Object[] result = (Object[]) it.next();
			System.out.println((String)result[0]);
			System.out.println((String)result[1]);
		}
	}
	
	/**
	 * Metoda weryfikująca czy w bazie istnieje użytkownik o padanym loginie i haśle.
	 * @param login nazwa użytkownika
	 * @param password hasło użytkownika
	 */
	@Override
	public boolean tryLogIn(String login, String password) {
		String q = "SELECT 1 FROM User u" +
				" WHERE u.login = :login " +
				" AND u.password = :password";
		Query query = manager.createQuery(q);
		query.setParameter("login", login);
		query.setParameter("password", HashAlghoritms.SHA1(password));
		try {
			query.getSingleResult();
		} catch (NoResultException e) {
			return false;
		} catch (NonUniqueResultException e) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Metoda weryfikująca czy dany użytkownik posiada konkretne uprawnienie.
	 * @deprecated metoda nie jest już wykorzystywana ze względu na zbyt dużą liczbę odwołań do bazy danych
	 */
	@Deprecated
	@Override
	public boolean isAuthorized(String login, int permissionNumber) {
		String q = "SELECT 1 " +
				" FROM User u JOIN u.groups g" +
				" JOIN g.roles r" +
				" JOIN r.permissions p" +
				" WHERE 1=1" +
				" AND u.login = :login" +
				" AND p.permissionNumber = :permissionNumber";
		Query query = manager.createQuery(q);
		query.setParameter("login", login);
		query.setParameter("permissionNumber", permissionNumber);
		
		@SuppressWarnings("rawtypes")
		Collection result = query.getResultList();
		if (result.size() >= 1)
			return true;
		else
			return false;
	}
	
	/**
	 * Metoda zwracająca wszystkie unikatowe kody uprawnień dostępne dla użytkownika.
	 * @param login nazwa użytkownika
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Integer> getUserPermissions(String login) {
		List<Integer> list = null;
		String q = "SELECT DISTINCT p.permissionNumber" +
				" FROM User u JOIN u.groups g" +
				" JOIN g.roles r" +
				" JOIN r.permissions p" +
				" WHERE 1=1" +
				" AND u.login = :login";
		Query query = manager.createQuery(q);
		query.setParameter("login", login);
		list = query.getResultList();
		
		return list;
	}
}
